/****************************************************************************
 ** Makou Reactor Final Fantasy VII Field Script Editor
 ** Copyright (C) 2009-2012 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/
#include "IsoArchive.h"

IsoArchive::IsoArchive(const QString &name)
{
	_file.setFileName(name);
}

IsoArchive::~IsoArchive()
{
}

quint8 IsoArchive::open(QIODevice::OpenMode mode)
{
	bool open = _file.open(mode);
	if(!open)	return 1;

	volume = volumeDescriptor();
//	qDebug() << volumeDescriptorToString(volume);
	qint64 size = _file.size();
	if(size%2352 != 0 || volume.vd1.volume_space_size != _file.size()/2352 || volume.vd1.id[0] != 'C' || volume.vd1.id[1] != 'D' || volume.vd1.id[2] != '0' || volume.vd1.id[3] != '0' || volume.vd1.id[4] != '1') {
		close();
		return 2;
	}
	
	dirVisisted.clear();
	directories = directoryRecord(volume.dr.drh.location_extent, volume.dr.drh.data_length);
//	pathTables1a = pathTable(volume.vd1.type_path_table, volume.vd1.path_table_size);
//	pathTables1b = pathTable(volume.vd1.opt_type_path_table, volume.vd1.path_table_size);
//	pathTables2a = pathTable(qFromBigEndian(volume.vd1.type_path_table2), volume.vd1.path_table_size);
//	pathTables2b = pathTable(qFromBigEndian(volume.vd1.opt_type_path_table2), volume.vd1.path_table_size);
	
	/*qDebug() << "PATHTABLE1";
	foreach(const PathTable &pt, pathTables1a)
		qDebug() << pathTableToString(pt);
	qDebug() << "PATHTABLE2";
	foreach(const PathTable &pt, pathTables1b)
		qDebug() << pathTableToString(pt);
	qDebug() << "PATHTABLE3";
	foreach(const PathTable &pt, pathTables2a)
		qDebug() << pathTableToString(pt, true);
	qDebug() << "PATHTABLE4";
	foreach(const PathTable &pt, pathTables2b)
		qDebug() << pathTableToString(pt, true);
	*/
	_file.reset();
	
	return 0;
}

void IsoArchive::close()
{
	_file.close();
}

qint64 IsoArchive::pos() const
{
	return isoPos(_file.pos());
}

qint64 IsoArchive::pos(QFile *file)
{
	return isoPos(file->pos());
}

qint64 IsoArchive::rawPos() const
{
	return _file.pos();
}

bool IsoArchive::seek(qint64 off)
{
	return _file.seek(filePos(off));
}

bool IsoArchive::seek(qint64 off, QFile *file)
{
	return file->seek(filePos(off));
}

bool IsoArchive::rawSeek(qint64 off)
{
	return _file.seek(off);
}

qint64 IsoArchive::size() const
{
	return isoPos(_file.size());
}

qint64 IsoArchive::size(QFile *file)
{
	return isoPos(file->size());
}

qint64 IsoArchive::rawSize() const
{
	return _file.size();
}

qint64 IsoArchive::isoPos(qint64 pos)
{
	qint64 start = (pos % 2352) - 24;
	if(start < 0) {
		start = 0;
	} else if(start > 2048) {
		start = 2048;
	}

	return 2048 * (pos / 2352) + start;
}

qint64 IsoArchive::filePos(qint64 pos)
{
	return 24 + (pos / 2048) * 2352 + pos % 2048;
}

QString IsoArchive::fileName()
{
	return _file.fileName();
}

qint64 IsoArchive::read(char *data, qint64 maxSize)
{
	qint64 pos = _file.pos(), read, readTotal = 0, seqLen;

	seqLen = qMin(2072 - (pos % 2352), maxSize);
	if(seqLen < 0)	return 0;

//	qDebug() << pos << "seek >> " << seqLen;
	if(!seek(isoPos(pos)))	return 0;

	while((read = _file.read(data, seqLen)) > 0) {
//		qDebug() << "read" << seqLen << read;
		data += read;
		maxSize -= read;
		readTotal += read;
		seqLen = qMin((qint64)2048, maxSize);
//		qDebug() << "seqLen" << seqLen << maxSize << _file.pos();
		// Si on est  la fin du secteur
		if(_file.pos() % 2352 >= 2072) {
			if(!_file.seek(_file.pos() + 304))	break;
//			qDebug() << "seek >> 304" << _file.pos();
		}
	}

	return read < 0 ? read : readTotal;
}

QByteArray IsoArchive::read(qint64 maxSize)
{
	char *data = new char[maxSize];

	QByteArray baData(data, read(data, maxSize));

	delete data;
	
	return baData;
}

qint64 IsoArchive::read(char *data, qint64 maxSize, QFile *file)
{
	qint64 pos = file->pos(), read, readTotal = 0, seqLen;

	seqLen = qMin(2072 - (pos % 2352), maxSize);

	if(!seek(isoPos(pos), file))	return 0;

	while((read = file->read(data, seqLen)) > 0) {
		data += read;
		maxSize -= read;
		readTotal += read;
		seqLen = qMin((qint64)2048, maxSize);
		// Si on est  la fin du secteur
		if(file->pos() % 2352 >= 2072) {
			if(!file->seek(file->pos() + 304))	break;
		}
	}

	return read < 0 ? read : readTotal;
}

QByteArray IsoArchive::read(qint64 maxSize, QFile *file)
{
	char *data = new char[maxSize];

	QByteArray baData(data, read(data, maxSize, file));

	delete data;

	return baData;
}

qint64 IsoArchive::write(char *data, qint64 maxSize)
{
	qint64 pos = _file.pos(), write, writeTotal = 0, seqLen;

	seqLen = qMin(2072 - (pos % 2352), maxSize);

//	qDebug() << pos << "seek >> " << seqLen;
	if(!seek(isoPos(pos)))	return 0;

	while((write = _file.write(data, seqLen)) > 0) {
//		qDebug() << "write" << seqLen << write;
		data += write;
		maxSize -= write;
		writeTotal += write;
		seqLen = qMin((qint64)2048, maxSize);
//		qDebug() << "seqLen" << seqLen << maxSize << _file.pos();
		// Si on est  la fin du secteur
		if(_file.pos() % 2352 >= 2072) {
			if(!_file.seek(_file.pos() + 304))	break;
//			qDebug() << "seek >> 304" << _file.pos();
		}
	}

	return write < 0 ? write : writeTotal;
}

qint64 IsoArchive::write(QByteArray &byteArray)
{
	return write(byteArray.data(), byteArray.size());
}

qint64 IsoArchive::write(char *data, qint64 maxSize, QFile *file)
{
	qint64 pos = file->pos(), write, writeTotal = 0, seqLen;

	seqLen = qMin(2072 - (pos % 2352), maxSize);

	if(!seek(isoPos(pos), file))	return 0;

	while((write = file->write(data, seqLen)) > 0) {
		data += write;
		maxSize -= write;
		writeTotal += write;
		seqLen = qMin((qint64)2048, maxSize);
		// Si on est  la fin du secteur
		if(file->pos() % 2352 >= 2072) {
			if(!file->seek(file->pos() + 304))	break;
		}
	}

	return write < 0 ? write : writeTotal;
}

qint64 IsoArchive::write(QByteArray &byteArray, QFile *file)
{
	return write(byteArray.data(), byteArray.size(), file);
}

qint64 IsoArchive::rawRead(char *data, qint64 maxSize)
{
	return _file.read(data, maxSize);
}

QByteArray IsoArchive::rawRead(qint64 maxSize)
{
	return _file.read(maxSize);
}

qint64 IsoArchive::rawWrite(char *data, qint64 maxSize)
{
	return _file.write(data, maxSize);
}

qint64 IsoArchive::rawWrite(QByteArray &byteArray)
{
	return _file.write(byteArray);
}

QByteArray IsoArchive::sectorHeader(quint32 num)
{
	_file.seek(2352 * num);
	return _file.read(24);
}

QByteArray IsoArchive::sectorFooter(quint32 num)
{
	_file.seek(2352 * num + 2072);
	return _file.read(280);
}

QByteArray IsoArchive::sector(quint32 num, quint16 maxSize)
{
	_file.seek(2352 * num + 24);
	return _file.read(maxSize);
}

quint32 IsoArchive::sectorCount()
{
	return volume.vd1.volume_space_size;
}

bool IsoArchive::seekToSector(quint32 num)
{
	return _file.seek(2352 * num);
}

QList<DirectoryRecord> IsoArchive::getDirectories()
{
	return directories;
}

/*QList<PathTable> IsoArchive::getPathTables1a()
{
	return pathTables1a;
}

QList<PathTable> IsoArchive::getPathTables1b()
{
	return pathTables1b;
}

QList<PathTable> IsoArchive::getPathTables2a()
{
	return pathTables2a;
}

QList<PathTable> IsoArchive::getPathTables2b()
{
	return pathTables2b;
}*/

VolumeDescriptor IsoArchive::volumeDescriptor(quint8 num)
{
//	qDebug() << "sizeof(VolumeDescriptor)" << sizeof(VolumeDescriptor1) << sizeof(VolumeDescriptor2) << sizeof(IsoTime) << sizeof(DirectoryRecord);
	VolumeDescriptor vd;

	_file.seek(2352 * (16 + num) + 24);
//	qint64 pos = _file.pos();
	_file.read((char *)&vd.vd1, sizeof(VolumeDescriptor1));
//	qint64 pos2 = _file.pos();
	_file.read((char *)&vd.dr.length_dr, 1);
	_file.read((char *)&vd.dr.extended_attr_record_length, 1);
	_file.peek((char *)&vd.dr.drh, sizeof(DirectoryRecordHead));
	_file.seek(_file.pos() + 31);
	vd.dr.name = _file.read(qMin((int)vd.dr.drh.length_fi, MAX_FILENAME_LENGTH));
	if(_file.pos() & 1) {
		_file.seek(_file.pos() + 1);// padding
	}
//	qDebug() << "sizeDirectoryRecord : " << (_file.pos() - pos2);
	_file.read((char *)&vd.vd2, sizeof(VolumeDescriptor2));

//	qDebug() << "size : " << (_file.pos() - pos);

	return vd;
}

QList<DirectoryRecord> IsoArchive::directoryRecord(quint32 sector, quint32 dataSize)
{
	QList<DirectoryRecord> directories;
	
	if(dirVisisted.contains(sector))	return directories;
	for(quint32 i=0 ; i<dataSize/2048 ; ++i)
		dirVisisted.append(sector + i);
	
	seek(2048 * sector);

	while(pos() < 2048 * sector + dataSize) {
		DirectoryRecord dr;
		qint64 beginPos = pos();
		dr.position = beginPos;
//		qDebug() << beginPos;

		read((char *)&dr.length_dr, 1);
		
		if(dr.length_dr == 0) {
			seek(2048 * ((beginPos / 2048) + 1));
			continue;
		}
		
		read((char *)&dr.extended_attr_record_length, 1);
		read((char *)&dr.drh, sizeof(DirectoryRecordHead));
		
		if(dr.drh.length_fi > MAX_FILENAME_LENGTH)	break;
		
		seek(beginPos + 33);
		
		dr.name = read(dr.drh.length_fi);
		dr.version = dr.name.mid(dr.name.indexOf(SEPARATOR_2)+1);
		dr.name = dr.name.left(dr.name.indexOf(SEPARATOR_2));
		seek(beginPos + dr.length_dr);
		
//		qDebug() << directoryRecordToString(dr);

		if((dr.drh.file_flags >> 1) & 1)// Directory
		{
			if(!dr.name.isEmpty() && dr.name != "\x01") {
//				qDebug() << "IN DIR" << dr.name << beginPos;
				dr.directories.append(directoryRecord(dr.drh.location_extent, dr.drh.data_length));
//				qDebug() << "OUT DIR" << dr.name;
				seek(beginPos + dr.length_dr);
			}/* else {
				qDebug() << dr.name << beginPos;
			}*/
		}
		else
		{
//			qDebug() << "FILE" << dr.name << beginPos;
		}
		
		directories.append(dr);
	}
	
	return directories;
}

QList<PathTable> IsoArchive::pathTable(quint32 sector, quint32 dataSize)
{
	QList<PathTable> pathTables;
	
	seek(2048 * sector);

	while(pos() < 2048 * sector + dataSize) {
		PathTable pt;
		pt.position = pos();

		read((char *)&pt.length_di, 1);
		read((char *)&pt.extended_attr_record_length, 1);
		read((char *)&pt.location_extent, 4);
		read((char *)&pt.parent_directory_number, 2);
		pt.name = read(pt.length_di);
		if(pt.length_di & 1) {
			seek(pos() + 1);
		}
		
		pathTables.append(pt);
	}
	
	return pathTables;
}

QByteArray IsoArchive::file(QString path, quint32 maxSize)
{
	DirectoryRecord fileRecord;
	
	if(!getDirectoryRecord(path, fileRecord)) {
		return QByteArray();
	}
	
	if((fileRecord.drh.file_flags >> 1) & 1)// Directory
	{
		return QByteArray();
	}
	
//	qDebug() << directoryRecordToString(fileRecord);
	
	seek(2048 * fileRecord.drh.location_extent);
	
	return read(maxSize == 0 ? fileRecord.drh.data_length : qMin(maxSize, fileRecord.drh.data_length));
}

bool IsoArchive::extract(QString path, QString destination, quint32 maxSize)
{
//	QTime t;t.start();
	DirectoryRecord fileRecord;
	
	if(!getDirectoryRecord(path, fileRecord)) {
		return false;
	}
	
	if((fileRecord.drh.file_flags >> 1) & 1)// Directory
	{
		return false;
	}
	
//	qDebug() << directoryRecordToString(fileRecord);
	
	maxSize = maxSize == 0 ? fileRecord.drh.data_length : qMin(maxSize, fileRecord.drh.data_length);
	char data[MAX_ISO_READ];
	qint64 r, totalR=0;
	QFile ret(destination);
	if(!ret.open(QIODevice::WriteOnly))		return false;
	
	seek(2048 * fileRecord.drh.location_extent);
	
	while(maxSize-totalR > 0 && (r = read(data, qMin((qint64)MAX_ISO_READ, maxSize-totalR))) > 0) {
		ret.write(data, r);
		totalR += r;
	}
	
//	qDebug() << "time" << t.elapsed();
	
	return true;
}

void IsoArchive::extractAll(QString destination)
{
	_extractAll(destination, directories);
}

void IsoArchive::_extractAll(QString destination, QList<DirectoryRecord> drs, QString currentInternalDir)
{
//	QTime t;t.start();

	QDir dir(destination);
	QString currentPath = dir.absolutePath().append('/');
//	qDebug() << currentPath;
	foreach(const DirectoryRecord &dr, drs) {
		QCoreApplication::processEvents();
		
		if((dr.drh.file_flags >> 1) & 1)// Directory
		{
			if(!dr.name.isEmpty() && dr.name != "\x01") {
				dir.mkdir(dr.name);
				_extractAll(currentPath + dr.name, dr.directories, currentInternalDir.isEmpty() ? dr.name : currentInternalDir + '/' + dr.name);
			}
		}
		else
		{
//			qDebug() << dr.name;
			extract(currentInternalDir.isEmpty() ? dr.name : currentInternalDir + '/' + dr.name, currentPath + dr.name);
		}
	}
	
//	qDebug() << "time" << t.elapsed();
}

bool IsoArchive::seekToFile(QString path)
{
	DirectoryRecord fileRecord;
	if(!getDirectoryRecord(path, fileRecord)) {
		return false;
	}
	
//	qDebug() << directoryRecordToString(fileRecord);
	
	return seek(2048 * fileRecord.drh.location_extent);
}

qint32 IsoArchive::diffCountSectors(QString path, quint32 newSize)
{
	DirectoryRecord fileRecord;
	if(!getDirectoryRecord(path, fileRecord)) {
		return false;
	}
	
	return (newSize - fileRecord.drh.data_length) / 2048;
}

bool IsoArchive::infosFile(QString path, quint32 &position, quint32 &size)
{
	DirectoryRecord fileRecord;
	if(!getDirectoryRecord(path, fileRecord)) {
		return false;
	}
	
	position = fileRecord.drh.location_extent;
	size = fileRecord.drh.data_length;
	return true;
}

//bool IsoArchive::insert(QString path, QString newISO, QByteArray data)
//{
//	QTime t;t.start();
//	DirectoryRecord fileRecord;
//	if(!getDirectoryRecord(path, fileRecord)) {
//		return false;
//	}
	
//	if((fileRecord.drh.file_flags >> 1) & 1)// Directory
//	{
//		return false;
//	}
	
//	qDebug() << directoryRecordToString(fileRecord);
	
//	seek(2048 * fileRecord.drh.location_extent);
	
//	qDebug() << "time" << t.elapsed();
	
//	return true;
//}

int IsoArchive::indexOfDirectoryRecord(QString name, QList<DirectoryRecord> directories)
{
	for(int i=0 ; i<directories.size() ; ++i) {
		if(directories.at(i).name == name) {
			return i;
		}
	}
	
	return -1;
}

bool IsoArchive::getDirectoryRecord(QString path, DirectoryRecord &fileRecord)
{
	QStringList dirNames = path.split('/', QString::SkipEmptyParts);
	int index = -1;
	QList<DirectoryRecord> directories = this->directories;
	if(dirNames.isEmpty()) {
		return false;
	}
	
	foreach(const QString &dirName, dirNames) {
		index = indexOfDirectoryRecord(dirName, directories);
		if(index == -1) {
			return false;
		}
		fileRecord = directories.at(index);
		directories = fileRecord.directories;
	}
	
	return true;
}

QString IsoArchive::isoTimeToString(IsoTime time)
{
	return QString("%1/%2/%3 %4:%5:%6 (%7ms) [GMT%8]").arg(
				QByteArray((char *)time.day, 2),
				QByteArray((char *)time.month, 2),
				QByteArray((char *)time.year, 4),
				QByteArray((char *)time.hour, 2),
				QByteArray((char *)time.minute, 2),
				QByteArray((char *)time.second, 2),
				QByteArray((char *)time.millis, 2),
				QString("%1%2").arg(time.GMT >= 0 ? "+" : "").arg(time.GMT));
}

QString IsoArchive::volumeDescriptorToString(VolumeDescriptor vd)
{
	QString out;
	out.append("struct VolumeDescriptor {\n");
	out.append(QString("\t type = %1\n").arg(vd.vd1.type, 2, 16, QChar('0')));
	out.append(QString("\t id = %1\n").arg(QString(QByteArray((char *)vd.vd1.id, sizeof(vd.vd1.id)))));
	out.append(QString("\t version = %1\n").arg(vd.vd1.version));
	if(vd.vd1.type != 0xff) {
		out.append(QString("\t system_id = %1\n").arg(QString(QByteArray((char *)vd.vd1.system_id, sizeof(vd.vd1.system_id)))));
		out.append(QString("\t volume_id = %1\n").arg(QString(QByteArray((char *)vd.vd1.volume_id, sizeof(vd.vd1.volume_id)))));
		out.append(QString("\t volume_space_size = %1\n").arg(vd.vd1.volume_space_size));
		out.append(QString("\t volume_space_size2 = %1\n").arg(qFromBigEndian(vd.vd1.volume_space_size2)));
		out.append(QString("\t volume_set_size = %1\n").arg(vd.vd1.volume_set_size));
		out.append(QString("\t volume_set_size2 = %1\n").arg(qFromBigEndian(vd.vd1.volume_set_size2)));
		out.append(QString("\t volume_sequence_number = %1\n").arg(vd.vd1.volume_sequence_number));
		out.append(QString("\t volume_sequence_number2 = %1\n").arg(qFromBigEndian(vd.vd1.volume_sequence_number2)));
		out.append(QString("\t logical_block_size = %1\n").arg(vd.vd1.logical_block_size));
		out.append(QString("\t logical_block_size2 = %1\n").arg(qFromBigEndian(vd.vd1.logical_block_size2)));
		out.append(QString("\t path_table_size = %1\n").arg(vd.vd1.path_table_size));
		out.append(QString("\t path_table_size2 = %1\n").arg(qFromBigEndian(vd.vd1.path_table_size2)));
		out.append(QString("\t type_path_table = %1\n").arg(vd.vd1.type_path_table));
		out.append(QString("\t opt_type_path_table = %1\n").arg(vd.vd1.opt_type_path_table));
		out.append(QString("\t type_path_table2 = %1\n").arg(qFromBigEndian(vd.vd1.type_path_table2)));
		out.append(QString("\t opt_type_path_table2 = %1\n").arg(qFromBigEndian(vd.vd1.opt_type_path_table2)));
		out.append("\t ====================\n");
		out.append(directoryRecordToString(vd.dr));
		out.append("\t ====================\n");
		out.append(QString("\t volume_set_id = %1\n").arg(QString(QByteArray((char *)vd.vd2.volume_set_id, sizeof(vd.vd2.volume_set_id)))));
		out.append(QString("\t publisher_id = %1\n").arg(QString(QByteArray((char *)vd.vd2.publisher_id, sizeof(vd.vd2.publisher_id)))));
		out.append(QString("\t preparer_id = %1\n").arg(QString(QByteArray((char *)vd.vd2.preparer_id, sizeof(vd.vd2.preparer_id)))));
		out.append(QString("\t application_id = %1\n").arg(QString(QByteArray((char *)vd.vd2.application_id, sizeof(vd.vd2.application_id)))));
		out.append(QString("\t copyright_file_id = %1\n").arg(QString(QByteArray((char *)vd.vd2.copyright_file_id, sizeof(vd.vd2.copyright_file_id)))));
		out.append(QString("\t abstract_file_id = %1\n").arg(QString(QByteArray((char *)vd.vd2.abstract_file_id, sizeof(vd.vd2.abstract_file_id)))));
		out.append(QString("\t bibliographic_file_id = %1\n").arg(QString(QByteArray((char *)vd.vd2.bibliographic_file_id, sizeof(vd.vd2.bibliographic_file_id)))));
		out.append(QString("\t creation_date = %1\n").arg(IsoArchive::isoTimeToString(vd.vd2.creation_date)));
		out.append(QString("\t file_structure_version = %1\n").arg(vd.vd2.file_structure_version));
		out.append(QString("\t application_data = %1\n").arg(QString(QByteArray((char *)vd.vd2.application_data, sizeof(vd.vd2.application_data)).toHex())));
	}
	out.append("}");

	return out;
}

QString IsoArchive::directoryRecordToString(DirectoryRecord dr)
{
	QString out;
	out.append("struct DirectoryRecord {\n");
	out.append(QString("\t length_dr = %1\n").arg(dr.length_dr));
	out.append(QString("\t extended_attr_record_length = %1\n").arg(dr.extended_attr_record_length));
	out.append(QString("\t location_extent = %1\n").arg(dr.drh.location_extent));
	out.append(QString("\t location_extent2 = %1\n").arg(qFromBigEndian(dr.drh.location_extent2)));
	out.append(QString("\t data_length = %1\n").arg(dr.drh.data_length));
	out.append(QString("\t data_length2 = %1\n").arg(qFromBigEndian(dr.drh.data_length2)));
	out.append(QString("\t %3/%2/%1 %4:%5:%6 [GMT%7]\n").arg(dr.drh.year, 2, 10, QChar('0')).arg(dr.drh.month, 2, 10, QChar('0')).arg(dr.drh.day, 2, 10, QChar('0')).arg(dr.drh.hour, 2, 10, QChar('0')).arg(dr.drh.minute, 2, 10, QChar('0')).arg(dr.drh.second, 2, 10, QChar('0')).arg(QString("%1%2").arg(dr.drh.GMT >= 0 ? "+" : "").arg(dr.drh.GMT)));
	out.append(QString("\t file_flags = %1\n").arg(dr.drh.file_flags, 8, 2, QChar('0')));
	out.append(QString("\t file_unit_size = %1\n").arg(dr.drh.file_unit_size));
	out.append(QString("\t interleave_grap_size = %1\n").arg(dr.drh.interleave_grap_size));
	out.append(QString("\t volume_sequence_number = %1\n").arg(dr.drh.volume_sequence_number));
	out.append(QString("\t volume_sequence_number2 = %1\n").arg(qFromBigEndian(dr.drh.volume_sequence_number2)));
	out.append(QString("\t length_fi = %1\n").arg(dr.drh.length_fi));
	out.append(QString("\t name = %1\n").arg(dr.name));
	out.append("}");
	return out;
}

QString IsoArchive::pathTableToString(PathTable pathTable, bool bigEndian)
{
	QString out;
	out.append("struct PathTable {\n");
	out.append(QString("\t length_di = %1\n").arg(pathTable.length_di));
	out.append(QString("\t extended_attr_record_length = %1\n").arg(pathTable.extended_attr_record_length));
	out.append(QString("\t location_extent = %1\n").arg(bigEndian ? qFromBigEndian(pathTable.location_extent) : pathTable.location_extent));
	out.append(QString("\t parent_directory_number = %1\n").arg(bigEndian ? qFromBigEndian(pathTable.parent_directory_number) : pathTable.parent_directory_number));
	out.append(QString("\t name = %1\n").arg(pathTable.name));
	out.append("}");
	return out;
}

quint8 IsoArchive::hex2Dec(quint8 hex)
{
	return 10*(hex/16) + hex%16;
}

quint8 IsoArchive::dec2Hex(quint8 dec)
{
	return 16*(dec/10) + dec%10;
}

QByteArray IsoArchive::int2Header(quint32 id)
{
	quint8 h1, h2, h3;

	if(id < 4350)
	{
		h1 = 0;
		h2 = IsoArchive::dec2Hex(id/75 + 2);
		h3 = IsoArchive::dec2Hex(id - 75*(IsoArchive::hex2Dec(h2) - 2));
	}
	else
	{
		h1 = IsoArchive::dec2Hex((id + 150) / 4500);
		h2 = IsoArchive::dec2Hex((id + 150 - IsoArchive::hex2Dec(h1)*4500) / 75);
		h3 = IsoArchive::dec2Hex(id + 150 - IsoArchive::hex2Dec(h1)*4500 - IsoArchive::hex2Dec(h2)*75);
	}
	
	return QByteArray().append((char)h1).append((char)h2).append((char)h3);
}

QByteArray IsoArchive::buildHeader(quint32 sector, quint8 type, quint8 mode)
{
	return QByteArray("\x00\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\x00", 12)
	        .append(int2Header(sector)).append((char)mode)
	        .append("\x00\x00", 2).append((char)type).append('\x00')
	        .append("\x00\x00", 2).append((char)type).append('\x00');
}

QByteArray IsoArchive::buildFooter(quint32 /*sector*/)
{
	//TODO (if possible): Checksum EDC/ECC (Error Detection Code & Error Correction Code)
	return QByteArray(280, '\x00');
}
