/****************************************************************************
 ** Makou Reactor Final Fantasy VII Field Script Editor
 ** Copyright (C) 2009-2012 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/
#include "Var.h"

int Var::load()
{
	//Chargement du fichier dans Var::var_names
	QFile fic(qApp->applicationDirPath()+"/vars.cfg");
	if(!fic.exists() && fic.open(QIODevice::WriteOnly))
	{
		fic.write(
			QString("1|3|$AerisLovePoints\r\n").toLocal8Bit()+
			QString("1|4|$TifaLovePoints\r\n").toLocal8Bit()+
			QString("1|5|$YuffieLovePoints\r\n").toLocal8Bit()+
			QString("1|6|$BarretLovePoints\r\n").toLocal8Bit()+
			QString("1|64|$KeyItems\r\n").toLocal8Bit()+
			QString("1|65|$KeyItems\r\n").toLocal8Bit()+
			QString("1|66|$KeyItems\r\n").toLocal8Bit()+
			QString("1|67|$KeyItems\r\n").toLocal8Bit()+
			QString("1|68|$KeyItems\r\n").toLocal8Bit()+
			QString("1|69|$KeyItems\r\n").toLocal8Bit()+
			QString("1|70|$KeyItems\r\n").toLocal8Bit()+
			QString("1|85|$ChocoboType1Catched\r\n").toLocal8Bit()+
			QString("1|86|$ChocoboType2Catched\r\n").toLocal8Bit()+
			QString("1|87|$ChocoboType3Catched\r\n").toLocal8Bit()+
			QString("1|88|$ChocoboType4Catched\r\n").toLocal8Bit()+
			QString("1|124|$ChocoboOnMap\r\n").toLocal8Bit()+
			QString("1|167|$VictoryFortCondor\r\n").toLocal8Bit()+
			QString("1|182|$DefeatFortCondor\r\n").toLocal8Bit()+
			QString("2|0|$GameMoment\r\n").toLocal8Bit()+
			QString("2|24|$BattleCount\r\n").toLocal8Bit()+
			QString("2|26|$BattleEscaped\r\n").toLocal8Bit()+
			QString("2|180|$GoldFortCondor\r\n").toLocal8Bit()+
			QString("3|9|$PartyLeader\r\n").toLocal8Bit()+
			QString("3|88|$NumStablesBought\r\n").toLocal8Bit()+
			QString("3|89|$NumStablesFilled\r\n").toLocal8Bit()+
			QString("13|31|$ErrorAddMateria\r\n").toLocal8Bit()+
			QString("13|84|$LastCloud\r\n").toLocal8Bit()+
			QString("13|85|$LastBarret\r\n").toLocal8Bit()+
			QString("13|86|$LastTifa\r\n").toLocal8Bit()+
			QString("13|87|$LastRedXIII\r\n").toLocal8Bit()+
			QString("13|88|$LastCid\r\n").toLocal8Bit()+
			QString("13|89|$LastYuffi\r\n").toLocal8Bit()+
			QString("13|90|$LastCaitSith\r\n").toLocal8Bit()+
			QString("13|92|$LastVincent\r\n").toLocal8Bit()
		);
		fic.close();
	}

	if(fic.open(QIODevice::ReadOnly))
	{
		QString line;
		QStringList liste;
		bool ok;
		int bank, adress;
		while(true)
		{
			line = QString(fic.readLine());
			if(line.isEmpty())	return 0;
			
			liste = line.split(QChar('|'));
			if(liste.size()!=3)	return 1;
			
			bank = liste.at(0).toInt(&ok);
			if(!ok)				return 1;
			if(bank<1 || bank>15)	continue;
			
			adress = liste.at(1).toInt(&ok);
			if(!ok)				return 1;
			if(adress<0 || adress>255)	continue;
			
			line = liste.at(2);
			if(line.isEmpty())	return 1;
			if(line.size()>50)	line = line.left(50);	
			
			Var::set(bank, adress, line.simplified());
		}
		fic.close();
		return 0;
	}
	return 1;
}

int Var::save(const QMap<quint16, QString> &new_var_names)
{
	//Enregistrement de Var::var_names dans le fichier
	QFile fic(qApp->applicationDirPath()+"/vars.cfg");
	if(fic.open(QIODevice::WriteOnly | QIODevice::Text))
	{
		Var::var_names = new_var_names;
		fic.setTextModeEnabled(true);
		QMapIterator<quint16, QString> i(Var::var_names);
		while(i.hasNext())
		{
			i.next();
			QString chaine = QString("%1|").arg(i.key() >> 8);
			chaine += QString("%1|").arg(i.key() & 255);
			chaine += i.value() + "\n";
			fic.write(QByteArray(chaine.toLocal8Bit()));
		}
		
		fic.close();
		return 0;
	}
	return 1;
}

QMap<quint16, QString> Var::var_names;

QString Var::name(quint8 bank, quint8 adress)
{
	return var_names.value(adress | (bank << 8), QString());//Retourne le nom de la variable s'il est list, retourne "" sinon
}

const QMap<quint16, QString> &Var::get()
{
	return var_names;
}

void Var::set(quint8 bank, quint8 adress, const QString &name)
{
	var_names.insert(adress | (bank << 8), name);
}

void Var::del(quint8 bank, quint8 adress)
{
	var_names.remove(adress | (bank << 8));
}

bool Var::exists(quint8 bank, quint8 adress)
{
	return var_names.contains(adress | (bank << 8));
}

QStringList Data::char_names;
QStringList Data::item_names;
QStringList Data::weapon_names;
QStringList Data::armor_names;
QStringList Data::accessory_names;
QStringList Data::materia_names;
const QList<Texte *> *Data::currentTextes;
QStringList Data::currentGrpScriptNames;
//QStringList Data::currentCharNames;
int Data::currentModelID=-1;
QStringList *Data::currentHrcNames=0;
QList<QStringList> *Data::currentAnimNames=0;
QStringList Data::field_names;
QStringList Data::movie_names;
QStringList Data::operateur_names;
QStringList Data::key_names;
QString Data::ff7DataPath_cache;
QString Data::ff7AppPath_cache;
QHash<QString, int> Data::charlgp_listPos;
QHash<QString, int> Data::charlgp_animBoneCount;

const QString &Data::ff7DataPath()
{
	if(ff7DataPath_cache.isNull()) {
		ff7DataPath_cache = QSettings("Square Soft, Inc.", "Final Fantasy VII").value("DataPath", "").toString();
		if(!ff7DataPath_cache.isEmpty())
			ff7DataPath_cache = QDir::cleanPath(ff7DataPath_cache)%"/";
	}
	return ff7DataPath_cache;
}

const QString &Data::ff7AppPath()
{
	if(ff7AppPath_cache.isNull()) {
		ff7AppPath_cache = QSettings("Square Soft, Inc.", "Final Fantasy VII").value("AppPath", "").toString();
		if(!ff7AppPath_cache.isEmpty())
			ff7AppPath_cache = QDir::cleanPath(ff7AppPath_cache)%"/";
	}
	return ff7AppPath_cache;
}

QString Data::charlgp_path()
{
	QString charPath = Config::value("charPath").toString();
	if(charPath.isEmpty()) {
		if(Data::ff7DataPath().isEmpty())	return QString();
		charPath = Data::ff7DataPath()%"/field/char.lgp";
	}

	return charPath;
}

void Data::charlgp_loadListPos(QFile *char_file)
{
	if(charlgp_listPos.isEmpty())
	{
		qint32 nbFiles;

		char_file->seek(12);
		char_file->read((char *)&nbFiles, 4);

		QString fileName;
		quint32 filePos;
		QByteArray charToc = char_file->read(27 * nbFiles);
		const char *charTocData = charToc.constData();
		charlgp_listPos.reserve(nbFiles);

		for(qint32 i=0 ; i<nbFiles ; ++i)
		{
			QCoreApplication::processEvents();
			fileName = charToc.mid(27 * i, 20);
			if(fileName.isEmpty())	return;
			memcpy(&filePos, &charTocData[27 * i + 20], 4);
			charlgp_listPos.insert(fileName.toLower(), filePos);
		}
	}
}

void Data::charlgp_loadAnimBoneCount()
{
	if(charlgp_animBoneCount.isEmpty())
	{
		QTime t;t.start();

		QString charPath = charlgp_path();
		if(!charPath.isEmpty()) {
			QFile char_file(charPath);
			if(char_file.open(QIODevice::ReadOnly)) {

				quint32 boneCount;

				QHashIterator<QString, int> i(charlgp_listPos);
				while(i.hasNext()) {
					i.next();
					if(i.key().endsWith(".a", Qt::CaseInsensitive)) {
						QCoreApplication::processEvents();
						char_file.seek(i.value()+32);
						char_file.read((char *)&boneCount, 4);
						charlgp_animBoneCount.insert(i.key().toLower(), boneCount);
					}
				}

				char_file.close();
			}
		}
		qDebug() << t.elapsed();
	}
}

int Data::load()
{
	/* Data::field_names
 << "BUG" << "World Map - Outside Midgar"
 << "World Map - Outside Kalm" << "World Map - Outside Chocobo Farm"
 << "World Map - Outside Mithryl Mine (Midgar side)" << "World Map - Outside Mithryl Mine (Junon side)"
 << "World Map - Outside Condor" << "World Map - Outside Junon"
 << "World Map - Outside Temple of the Ancients" << "World Map - Outside Old Man's house (receive Mithryl)"
 << "World Map - Outside Gongaga blaksmith (?)" << "World Map - Outside Mideel"
 << "World Map - Outside Materia Cave (Quadra Magic)" << "World Map - Outside Costa Del Sol"
 << "World Map - Outside Mt. Coral" << "World Map - Outside North Coral"
 << "World Map - Outside Gold Saucer (In desert, can't move)" << "World Map - Jungle, Outside blown reactor (Turks)"
 << "World Map - Outside Cosmo Canyon" << "World Map - Outside Nibelheim (Town Side)"
 << "World Map - Outside Rocket Town (South Side)" << "World Map - Outside Rocket Town (South Side)"
 << "World Map - Outside Materia Cave (HP<->MP Materia)" << "World Map - Outside Wutai"
 << "World Map - Outside Materia Cave (Mime Materia)" << "World Map - Outside Bone Village"
 << "World Map - Outside Corel Valley Cave (Icicle Area)" << "World Map - Outside Icicle Inn (South Side)"
 << "World Map - Outside Mystery House" << "World Map - Outside Materia Cave (Knights of the Round Materia)"
 << "World Map - Underwater - Costa del Sol" << "World Map - Underwater - North Area"
 << "World Map - 0,0,0 (Northwest corner)" << "World Map - 0,0,0 (Northwest corner)"
 << "World Map - 0,0,0 (Northwest corner)" << "World Map - 0,0,0 (Northwest corner)"
 << "World Map Event - Ship pulls out of Junon" << "World Map Event - Ship pulls into Costa Del Sol"
 << "World Map Event - Ship goes from Junon to Costa Del Sol" << "World Map Event - Ship goes from Costa Del Sol to Junon"
 << "World Map - Little Bronco Crash start" << "World Map - Junon, On the water"
 << "World Map - Underwater - North Area" << "World Map - Outside Nibelheim (Mt. Nibel Side)"
 << "World Map - Outside Mt. Nibel (Nibelheim Side)" << "World Map - 0,0,0 (Northwest corner)"
 << "World Map - Outside Mt. Nibel (West Side)" << "World Map - Outside Icicle Inn (North Side)"
 << "World Map - Great Glacier" << "World Map - Outside Rocket Town (North Side)"
 << "World Map - 0,0,0 (Northwest corner)" << "World Map - 0,0,0 (Northwest corner)"
 << "World Map - 0,0,0 (Northwest corner)" << "World Map - 0,0,0 (Northwest corner)"
 << "World Map - Junon, On the water" << "World Map - Outside Ancient Forset"
 << "World Map - Underwater - North Area" << "World Map - Valley south of City of Ancients"
 << "World Map - Valley, City of Ancients enterance" << "World Map - 0,0,0 (Northwest corner)"
 << "World Map - Snow Maze" << "World Map - Snow Maze"
 << "World Map - Snow Maze" << "World Map - Snow Maze"
 << "World Map - Snow Maze - Outside Matera Cave (All)"*/
	if(char_names.isEmpty()) {
		char_names
				<< QObject::tr("Clad") << QObject::tr("Barret") << QObject::tr("Tifa")
				<< QObject::tr("Aeris") << QObject::tr("Red XIII") << QObject::tr("Youfie")
				<< QObject::tr("Cait Sith") << QObject::tr("Vincent") << QObject::tr("Cid")
				<< QObject::tr("Jeune Clad") << QObject::tr("Sephiroth") << QObject::tr("Chocobo");
	}

	if(operateur_names.isEmpty()) {
		operateur_names
				<< "==" << "!=" << ">" << "<" << ">=" << "<=" << "&" << "^" << "|" << "& (1<<" << "^ (1<<";
	}
	
	if(key_names.isEmpty()) {
		key_names
				<< QObject::tr("[CAMERA|L2]") << QObject::tr("[CIBLE|R2]")
				<< QObject::tr("[PAGE HAUT|L1]") << QObject::tr("[PAGE BAS|R1]")
				<< QObject::tr("[MENU|TRIANGLE]") << QObject::tr("[OK|ROND]")
				<< QObject::tr("[ANNULER|CROIX]") << QObject::tr("[CHANGER|CARRE]")
				<< QObject::tr("[ASSISTER|SELECT]") << QString("[???]")
				<< QString("[???]") << QObject::tr("[DEMARRER|START]")
				<< QObject::tr("[HAUT]") << QObject::tr("[DROITE]")
				<< QObject::tr("[BAS]") << QObject::tr("[GAUCHE]");
	}
	
	if(movie_names.isEmpty()) {
		movie_names
				<< "fship2" << "fship2n" << "d_ropego" << "d_ropein" << "u_ropein" << "u_ropego"
				<< "gold2" << "gold3" << "gold4" << "gold6" << "gold5"
				<< "boogup" << "boogdown" << "junair_u" << "junair_d" << "junelein" << "junelego" << "junin_in" << "junin_go"
				<< "moriya" << "mkup" << "northmk" << "mk8" << "ontrain" << "mainplr" << "smk" << "southmk" << "plrexp" << "fallpl"
				<< "monitor" << "bike" << "mtnvl" << "mtnvl2" << "brgnvl" << "nvlmk" << "nivlsfs" << "jenova_e"
				<< "junon" << "hiwind0" << "mtcrl" << "gold1" << "biskdead" << "boogdemo" << "boogstar" << "setogake"
				<< "rcktfail" << "jairofly" << "jairofal" << "gold7" << "gold7_2" << "earithdd" << "funeral"
				<< "car_1209" << "opening" << "greatpit" << "c_scene1" << "c_scene2" << "c_scene3" << "biglight"
				<< "meteosky" << "weapon0" << "weapon1" << "weapon2" << "weapon3" << "weapon4" << "weapon5" << "hwindfly"
				<< "phoenix" << "nrcrl" << "nrcrl_b" << "dumcrush" << "zmind01" << "zmind02" << "zmind03"
				<< "gelnica" << "rcketoff" << "white2" << "junsea" << "rckethit0" << "rckethit1" << "meteofix"
				<< "canonon" << "feelwin0" << "feelwin1" << "canonht1" << "canonht2" <<"canonh3f"
				<< "parashot" << "hwindjet" << "canonht0" << "wh2e2" << "loslake1" << "lslmv" << "canonh1p"
				<< "canon" << "" << "last4_2" << "last4_3" << "last4_4" << "lastmap" << "lastflor"
				<< "ending1" << "ending3" << "fcar" << "white2" << "Ending2";
	}
	
	QString path = Config::value("kernel2Path", QString()).toString();
	if(path.isEmpty()) {
		path = ff7DataPath();
		if(path.isEmpty())	return 2;
		path.append("kernel/kernel2.bin");
	}
	
	QFile fic(path);
	if(fic.exists() && fic.open(QIODevice::ReadOnly))
	{
		quint32 fileSize;
		fic.read((char *)&fileSize, 4);
		if(fileSize+4!=fic.size())	return 2;
		const QByteArray &data = LZS::decompress(fic.read(fileSize));
		const char *constData = data.constData();
		int dataSize = data.size();
		fic.close();
		int pos=0;
		for(int i=0 ; i<10 ; ++i)
		{
			memcpy(&fileSize, &constData[pos], 4);
			pos += fileSize + 4;
			if(pos > dataSize)	return 1;
		}
		memcpy(&fileSize, &constData[pos], 4);
		pos += 4;
		if((int)(pos+fileSize) > dataSize)	return 1;
		item_names.clear();
		fill(data.mid(pos, fileSize), item_names);
		pos += fileSize;
		
		if(pos+4 > dataSize)	return 1;
		memcpy(&fileSize, &constData[pos], 4);
		pos += 4;
		if((int)(pos+fileSize) > dataSize)	return 1;
		weapon_names.clear();
		fill(data.mid(pos, fileSize), weapon_names);
		pos += fileSize;
		
		if(pos+4 > dataSize)	return 1;
		memcpy(&fileSize, &constData[pos], 4);
		pos += 4;
		if((int)(pos+fileSize) > dataSize)	return 1;
		armor_names.clear();
		fill(data.mid(pos, fileSize), armor_names);
		pos += fileSize;
		
		if((int)(pos+4) > dataSize)	return 1;
		memcpy(&fileSize, &constData[pos], 4);
		pos += 4;
		if((int)(pos+fileSize) > dataSize)	return 1;
		accessory_names.clear();
		fill(data.mid(pos, fileSize), accessory_names);
		pos += fileSize;
		
		if((int)(pos+4) > dataSize)	return 1;
		memcpy(&fileSize, &constData[pos], 4);
		pos += 4;
		if((int)(pos+fileSize) > dataSize)	return 1;
		materia_names.clear();
		fill(data.mid(pos, fileSize), materia_names);
		// pos += fileSize;
		
		/* if(pos+4 > dataSize)	return;
  memcpy(&fileSize, &constData[pos], 4);
  pos += 4;
  if(pos+fileSize > dataSize)	return;
  Data::fill(data.mid(pos, fileSize), Data::key_item_names); */
	}
	return 0;
}

void Data::fill(const QByteArray &data, QStringList &names)
{
	bool jp = Config::value("jp_txt", false).toBool();
	Config::setValue("jp_txt", false);
	int i=0, dataSize = data.size();
	quint16 start=0;
	while(i*2+2 < dataSize)
	{
		memcpy(&start, data.mid(i*2,2), 2);
		if(start >= dataSize)	break;
		names.append(Texte(data.mid(start)).getText(true));
		++i;
	}
	Config::setValue("jp_txt", jp);
}

void Data::openMaplist(const QByteArray &data)
{
	if(data.size()<2)	return;
	quint16 nbMap;
	memcpy(&nbMap, data.mid(0,2), 2);
	if(data.size()!=2+nbMap*32)	return;
	field_names.clear();
	for(int i=0 ; i<nbMap ; ++i)
	{
		field_names.append(QString(data.mid(2+i*32, 32)).simplified());
	}
}

// Standard mapList
void Data::openMaplist(bool PC)
{
	field_names.clear();
	field_names << "dummy" << "wm0" << "wm1" << "wm2" << "wm3" << "wm4" << "wm5" << "wm6"
				<< "wm7" << "wm8" << "wm9" << "wm10" << "wm11" << "wm12" << "wm13" << "wm14"
				<< "wm15" << "wm16" << "wm17" << "wm18" << "wm19" << "wm20" << "wm21" << "wm22"
				<< "wm23" << "wm24" << "wm25" << "wm26" << "wm27" << "wm28" << "wm29" << "wm30"
				<< "wm31" << "wm32" << "wm33" << "wm34" << "wm35" << "wm36" << "wm37" << "wm38"
				<< "wm39" << "wm40" << "wm41" << "wm42" << "wm43" << "wm44" << "wm45" << "wm46"
				<< "wm47" << "wm48" << "wm49" << "wm50" << "wm51" << "wm52" << "wm53" << "wm54"
				<< "wm55" << "wm56" << "wm57" << "wm58" << "wm59" << "wm60" << "wm61" << "wm62"
				<< "wm63" << "startmap" << "fship_1" << "fship_12" << "fship_2" << "fship_22"
				<< "fship_23" << "fship_24" << "fship_25" << "fship_3" << "fship_4" << "fship_42"
				<< "fship_5" << "hill" << "zz1" << "zz2" << "zz3" << "zz4" << "zz5" << "zz6"
				<< "zz7" << "zz8" << "sea" << "sky" << "q_1"/*qa*/ << "q_2"/*qb*/ << "q_3"/*qc*/ << "q_4"/*qd*/ << "q_5"/*qe*/
				<< "blackbg1" << "blackbg2" << "blackbg3" << "blackbg4" << "blackbg5" << "blackbg6"
				<< "blackbg7" << "blackbg8" << "blackbg9" << "blackbga" << "blackbgb" << "blackbgc"
				<< "blackbgd" << "blackbge" << "blackbgf" << "blackbgg" << "blackbgh" << "blackbgi"
				<< "blackbgj" << "blackbgk" << "whitebg1" << "whitebg2" << "whitebg3" << "md1stin"
				<< "md1_1" << "md1_2" << "nrthmk" << "nmkin_1" << "elevtr1" << "nmkin_2" << "nmkin_3"
				<< "nmkin_4" << "nmkin_5" << "southmk1" << "southmk2" << "smkin_1" << "smkin_2"
				<< "smkin_3" << "smkin_4" << "smkin_5" << "md8_1" << "md8_2" << "md8_3" << "md8_4"
				<< "md8brdg" << "cargoin" << "tin_1" << "tin_2" << "tin_3" << "tin_4" << "rootmap"
				<< "mds7st1" << "mds7st2" << "mds7st3" << "mds7st32" << "mds7_w1" << "mds7_w2"
				<< "mds7_w3" << "mds7" << "mds7_im" << "7min1"/*min71*/ << "mds7pb_1" << "mds7pb_2"
				<< "mds7plr1" << "mds7plr2" << "pillar_1" << "pillar_2" << "pillar_3" << "tunnel_1"
				<< "tunnel_2" << "tunnel_3" << "4sbwy_1"/*sbwy4_1*/ << "4sbwy_2"/*sbwy4_2*/ << "4sbwy_3"/*sbwy4_3*/ << "4sbwy_4"/*sbwy4_4*/
				<< "4sbwy_5"/*sbwy4_5*/ << "4sbwy_6"/*sbwy4_6*/ << "mds5_5" << "mds5_4" << "mds5_3" << "mds5_2"
				<< "5min1_1"/*min51_1*/ << "5min1_2"/*min51_2*/ << "mds5_dk" << "mds5_1" << "mds5_w" << "mds5_i"
				<< "mds5_m" << "church" << "chrin_1a" << "chrin_1b" << "chrin_2" << "chrin_3a"
				<< "chrin_3b" << "eals_1" << "ealin_1" << "ealin_12" << "ealin_2" << "mds6_1" << "mds6_2"
				<< "mds6_22" << "mds6_3" << "mrkt2" << "mkt_w" << "mkt_mens" << "mkt_ia" << "mktinn"
				<< "mkt_m" << "mkt_s1" << "mkt_s2" << "mkt_s3" << "mktpb" << "mrkt1" << "colne_1"
				<< "colne_2" << "colne_3" << "colne_4" << "colne_5" << "colne_6" << "colne_b1"
				<< "colne_b3" << "mrkt3" << "onna_1" << "onna_2" << "onna_3" << "onna_4" << "onna_5"
				<< "onna_52" << "onna_6" << "mrkt4" << "wcrimb_1" << "wcrimb_2" << "md0" << "roadend"
				<< "sinbil_1" << "sinbil_2" << "blinst_1" << "blinst_2" << "blinst_3" << "blinele"
				<< "eleout" << "blin1" << "blin2" << "blin2_i" << "blin3_1" << "blin59" << "blin60_1"
				<< "blin60_2" << "blin61" << "blin62_1" << "blin62_2" << "blin62_3" << "blin63_1"
				<< "blin63_t" << "blin64" << "blin65_1" << "blin65_2" << "blin66_1" << "blin66_2"
				<< "blin66_3" << "blin66_4" << "blin66_5" << "blin66_6" << "blin67_1" << "blin671b"
				<< "blin67_2" << "blin67_3" << "blin673b" << "blin67_4" << "blin68_1" << "blin68_2"
				<< "blin69_1" << "blin69_2" << "blin70_1" << "blin70_2" << "blin70_3" << "blin70_4"
				<< "niv_w" << "nvmin1_1" << "nvmin1_2" << "nivinn_1" << "nivinn_2" << "nivinn_3"
				<< "niv_cl" << "trackin" << "trackin2" << "nivgate" << "nivgate2" << "nivgate3"
				<< "nivl" << "nivl_2" << "nivl_3" << "nivl_4" << "niv_ti1" << "niv_ti2" << "niv_ti3"
				<< "niv_ti4" << "nivl_b1" << "nivl_b12" << "nivl_b2" << "nivl_b22" << "nivl_e1"
				<< "nivl_e2" << "nivl_e3" << "sinin1_1" << "sinin1_2" << "sinin2_1" << "sinin2_2"
				<< "sinin3" << "sininb1" << "sininb2" << "sininb31" << "sininb32" << "sininb33"
				<< "sininb41" << "sininb42" << "sininb51" << "sininb52" << "mtnvl2" << "mtnvl3"
				<< "mtnvl4" << "mtnvl5" << "mtnvl6" << "mtnvl6b" << "nvdun1" << "nvdun2" << "nvdun3"
				<< "nvdun31" << "nvdun4" << "nvmkin1" << "nvmkin21" << "nvmkin22" << "nvmkin23"
				<< "nvmkin31" << "nvmkin32" << "elm_wa" << "elm_i" << "elmpb" << "elminn_1" << "elminn_2"
				<< "elmin1_1" << "elmin1_2" << "elm" << "elmin2_1" << "elmin2_2" << "elmin3_1"
				<< "elmin3_2" << "elmtow" << "elmin4_1" << "elmin4_2" << "farm" << "frmin" << "frcyo"
				<< "trap" << "fr_e" << "sichi" << "psdun_1" << "psdun_2" << "psdun_3" << "psdun_4"
				<< "condor1" << "condor2" << "convil_1" << "convil_2" << "convil_3" << "convil_4"
				<< "junon" << "junonr1" << "junonr2" << "junonr3" << "junonr4" << "jun_wa" << "jun_i1"
				<< "jun_m" << "junmin1" << "junmin2" << "junmin3" << "junonl1" << "junonl2" << "junonl3"
				<< "jun_w" << "jun_a" << "jun_i2" << "juninn" << "junpb_1" << "junpb_2" << "junpb_3"
				<< "junmin4" << "junmin5" << "jundoc1a" << "jundoc1b" << "junair" << "junair2"
				<< "junin1" << "junin1a" << "junele1" << "junin2" << "junin3" << "junele2"
				<< "junin4" << "junin5" << "junin6" << "junin7" << "junbin1" << "junbin12"
				<< "junbin21" << "junbin22" << "junbin3" << "junbin4" << "junbin5" << "junmon"
				<< "junsbd1" << "subin_1a" << "subin_1b" << "subin_2a" << "subin_2b" << "subin_3"
				<< "subin_4" << "junone2" << "junone3" << "junone4" << "junone5" << "junone6" << "junone7"
				<< "spgate" << "spipe_1" << "spipe_2" << "semkin_1" << "semkin_2" << "semkin_8" << "semkin_3"
				<< "semkin_4" << "semkin_5" << "semkin_6" << "semkin_7" << "ujunon1" << "ujunon2" << "ujunon3"
				<< "prisila" << "ujun_w" << "jumin" << "ujunon4" << "ujunon5" << "ship_1" << "ship_2" << "shpin_22"
				<< "shpin_2" << "shpin_3" << "del1" << "del12" << "del2" << "delinn" << "delpb" << "delmin1"
				<< "delmin12" << "delmin2" << "del3" << "ncorel" << "ncorel2" << "ncorel3" << "ncoin1"
				<< "ncoin2" << "ncoin3" << "ncoinn" << "ropest" << "mtcrl_0" << "mtcrl_1" << "mtcrl_2"
				<< "mtcrl_3" << "mtcrl_4" << "mtcrl_5" << "mtcrl_6" << "mtcrl_7" << "mtcrl_8" << "mtcrl_9"
				<< "corel1" << "corel2" << "corel3" << "jail1" << "jailin1" << "jail2" << "jailpb" << "jailin2"
				<< "jailin3" << "jailin4" << "jail3" << "jail4" << "dyne" << "desert1" << "desert2" << "corelin"
				<< "astage_a" << "astage_b" << "jet" << "jetin1" << "bigwheel" << "bwhlin" << "bwhlin2" << "ghotel"
				<< "ghotin_1" << "ghotin_4" << "ghotin_2" << "ghotin_3" << "gldst" << "gldgate" << "gldinfo"
				<< "coloss" << "coloin1" << "coloin2" << "clsin2_1" << "clsin2_2" << "clsin2_3" << "games" << "games_1"
				<< "games_2" << "mogu_1" << "chorace" << "chorace2" << "crcin_1" << "crcin_2" << "gldelev" << "gonjun1"
				<< "gonjun2" << "gnmkf" << "gnmk" << "gongaga" << "gon_wa1" << "gon_wa2" << "gon_i" << "gninn" << "gomin"
				<< "goson" << "cos_btm" << "cos_btm2" << "cosmo" << "cosmo2" << "cosin1" << "cosin1_1" << "cosin2"
				<< "cosin3" << "cosin4" << "cosin5" << "cosmin2" << "cosmin3" << "cosmin4" << "cosmin6" << "cosmin7"
				<< "cos_top" << "bugin1a" << "bugin1b" << "bugin1c" << "bugin2" << "bugin3" << "gidun_1" << "gidun_2"
				<< "gidun_4" << "gidun_3" << "seto1" << "rckt2" << "rckt3" << "rkt_w" << "rkt_i" << "rktinn1" << "rktinn2"
				<< "rckt" << "rktsid" << "rktmin1" << "rktmin2" << "rcktbas1" << "rcktbas2" << "rcktin1" << "rcktin2"
				<< "rcktin3" << "rcktin4" << "rcktin5" << "rcktin6" << "rcktin7" << "rcktin8" << "pass" << "yougan"
				<< "yougan2" << "yougan3" << "uta_wa" << "uta_im" << "utmin1" << "utmin2" << "uutai1" << "utapb"
				<< "yufy1" << "yufy2" << "hideway1" << "hideway2" << "hideway3" << "5tower"/*tower5*/ << "uutai2" << "uttmpin1"
				<< "uttmpin2" << "uttmpin3" << "uttmpin4" << "datiao_1" << "datiao_2" << "datiao_3" << "datiao_4"
				<< "datiao_5" << "datiao_6" << "datiao_7" << "datiao_8" << "jtempl" << "jtemplb" << "jtmpin1"
				<< "jtmpin2" << "kuro_1" << "kuro_2" << "kuro_3" << "kuro_4" << "kuro_5" << "kuro_6" << "kuro_7"
				<< "kuro_8" << "kuro_82" << "kuro_9" << "kuro_10" << "kuro_11" << "kuro_12" << "bonevil" << "slfrst_1"
				<< "slfrst_2" << "anfrst_1" << "anfrst_2" << "anfrst_3" << "anfrst_4" << "anfrst_5" << "sango1"
				<< "sango2" << "sango3" << "sandun_1" << "sandun_2" << "lost1" << "losin1" << "losin2" << "losin3"
				<< "lost2" << "lost3" << "losinn" << "loslake1" << "loslake2" << "loslake3" << "blue_1" << "blue_2"
				<< "white1" << "white2" << "hekiga" << "whitein" << "ancnt1" << "ancnt2" << "ancnt3" << "ancnt4"
				<< "snw_w" << "sninn_1" << "sninn_2" << "sninn_b1" << "snow" << "snmin1" << "snmin2" << "snmayor"
				<< "hyou1" << "hyou2" << "hyou3" << "icedun_1" << "icedun_2" << "hyou4" << "hyou5_1" << "hyou5_2"
				<< "hyou5_3" << "hyou5_4" << "hyou6" << "hyoumap" << "move_s" << "move_i" << "move_f" << "move_r"
				<< "move_u" << "move_d" << "hyou7" << "hyou8_1" << "hyou8_2" << "hyou9" << "hyou10" << "hyou11"
				<< "hyou12" << "hyou13_1" << "hyou13_2" << "hyou14" << "gaiafoot" << "holu_1" << "holu_2"
				<< "gaia_1" << "gaiin_1" << "gaiin_2" << "gaia_2" << "gaiin_3" << "gaia_31" << "gaia_32"
				<< "gaiin_4" << "gaiin_5" << "gaiin_6" << "gaiin_7" << "crater_1" << "crater_2" << "trnad_1"
				<< "trnad_2" << "trnad_3" << "trnad_4" << "trnad_51" << "trnad_52" << "trnad_53" << "woa_1"
				<< "woa_2" << "woa_3" << "itown1a" << "itown12" << "itown1b" << "itown2" << "ithill" << "itown_w"
				<< "itown_i" << "itown_m" << "ithos" << "itmin1" << "itmin2" << "life" << "life2" << "zmind1"
				<< "zmind2" << "zmind3" << "zcoal_1" << "zcoal_2" << "zcoal_3" << "md8_5" << "md8_6" << "md8_b1"
				<< "md8_b2" << "4sbwy_22"/*sbwy4_22*/ << "tunnel_4" << "tunnel_5" << "md8brdg2" << "md8_32" << "canon_1"
				<< "canon_2" << "md_e1" << "xmvtes" << "las0_1" << "las0_2" << "las0_3" << "las0_4" << "las0_5"
				<< "las0_6" << "las0_7" << "las0_8" << "las1_1" << "las1_2" << "las1_3" << "las1_4" << "las2_1"
				<< "las2_2" << "las2_3" << "las2_4" << "las3_1" << "las3_2" << "las3_3" << "las4_0" << "las4_1"
				<< "las4_2" << "las4_3" << "las4_4" << "lastmap" << "fallp" << "m_endo" << "hill2" << "bonevil2"
				<< "junone22" << "rckt32" << "jtemplc" << "fship_26" << "las4_42" << "tunnel_6" << "md8_52" << "sininb34"
				<< "mds7st33" << "midgal" << "sininb35" << "nivgate4" << "sininb36" << "ztruck";

	if(PC) {
		field_names[88] = "qa";
		field_names[89] = "qb";
		field_names[90] = "qc";
		field_names[91] = "qd";
		field_names[92] = "qe";
		field_names[153] = "min71";
		field_names[164] = "sbwy4_1";
		field_names[165] = "sbwy4_2";
		field_names[166] = "sbwy4_3";
		field_names[167] = "sbwy4_4";
		field_names[168] = "sbwy4_5";
		field_names[169] = "sbwy4_6";
		field_names[174] = "min51_1";
		field_names[175] = "min51_2";
		field_names[586] = "tower5";
		field_names[735] = "sbwy4_22";
	}
}
