/****************************************************************************
 ** Makou Reactor Final Fantasy VII Field Script Editor
 ** Copyright (C) 2009-2012 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/
#ifndef DEF_WINDOW
#define DEF_WINDOW

#define DIR_UP		0
#define DIR_DOWN	1

#include <QtGui>
#include "FieldArchive.h"
#include "GrpScript.h"
#include "VarManager.h"
#include "TextManager.h"
#include "ApercuBG.h"
#include "Search.h"
#include "GrpScriptList.h"
#include "ScriptList.h"
#include "CommandeList.h"
#include "ModelManager.h"
#include "FieldModel.h"
#include "WalkmeshManager.h"
#include "ConfigWindow.h"
#include "EncounterWidget.h"
#include "TutWidget.h"
#include "MiscWidget.h"
#include "ImportDialog.h"

class Window : public QMainWindow
{
    Q_OBJECT
public:
	Window();
	~Window();

	void ouvrir(const QString &cheminFic, bool isDir=false);
	FieldArchive::Sorting getFieldSorting();
public slots:
	void ouvrirFichier();
	void ouvrirDossier();
	void refresh();

	void activerSave(bool enabled=true);
	void enregistrerSous(bool currentPath=false);
	void enregistrer();
	int fermer(bool quit=false);

	void gotoCommande(int fileID, int grpScriptID, int scriptID, int commandeID);

	void jp_txt(bool);

	//	void notifyFileChanged(const QString &path);
	//	void notifyDirectoryChanged(const QString &path);
	void exporter();
	void importer();
	void varManager();
	void runFF7();
	void textManager();
	void modelManager();
	void encounterManager();
	void tutManager();
	void walkmeshManager();
	void backgroundManager();
	void miscManager();
	void searchManager();
	void a_propos();
private slots:
	void emitCommandeID();
	void changeHistoric(const Historic &);
	void undo();
	void redo();

	void ouvrirField();
	void afficherGrpScripts();
	void afficherScripts();
	void filterMap();
	void change_language(QAction *);
	void config();
signals:
	void fileIDChanged(int);
	void grpScriptIDChanged(int);
	void scriptIDChanged(int);
	void commandeIDChanged(int);
private:
	QLineEdit *lineSearch;
	QTreeWidget *liste;
	GrpScriptList *liste2;
	ScriptList *liste3;
	CommandeList *zoneScript;
	QStackedWidget *zonePreview;
	ApercuBG *zoneImage;
	FieldModel *fieldModel;
	QGridLayout *gridLayout;

	QToolBar *toolBar;

	FieldArchive *fieldArchive;
	Field *field;
	bool firstShow;

	Search *searchDialog;
	VarManager *varDialog;

	QAction *actionSave;
	QAction *actionSaveAs;
	QAction *actionExport;
	QAction *actionImport;
	QAction *actionClose;
	QAction *actionFind;
	QAction *actionText;
	QAction *actionModels;
	QAction *actionEncounter;
	QAction *actionTut;
	QAction *actionWalkmesh;
	QAction *actionBackground;
	QAction *actionMisc;
	QAction *actionJp_txt;
	QAction *actionUndo;
	QAction *actionRedo;
	QMenu *menuLang;

	TextManager *textDialog;

	QProgressBar *progression;
	QLabel *auteurLbl;

	void restart_now();
	QStack<Historic> hists;
	QStack<Historic> restoreHists;
	int currentHistPos;
protected:
	void showEvent(QShowEvent *);
	void closeEvent(QCloseEvent *);
	QMenu *createPopupMenu();
};

#endif
